using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlTypes;
using System.Data.SqlClient;
using System.Data.Common;
using System.Windows.Forms;
using System.Globalization;

namespace ParametersDemo
{
    class ParamDemo
    {
        static private string SqlServer;
        static private string SqlLogin = "";
        static private string SqlPassword;
        static private bool Parameterize = true;
        static private uint QueryIterations = 200;
        static private SqlConnection cn = new SqlConnection();
        private void ParamDemo() { } 
        
        static int Main(string[] args)
        {
            try
            {
                if (!CheckCommandLine(args)) return 1;
                if (!ConnectToSql()) return 2;
                FreeProcCache();
                if (Parameterize)
                    RunQueriesParameterized();
                else
                    RunQueriesNotParameterized();
                return 0;
            }
            catch (Exception ex)
            {
                Console.WriteLine("Unexpected exception: \n{0}", ex.ToString());
                return 2;
            }
        }

        static private bool RunQueriesParameterized()
        {
            SqlCommand cmd = new SqlCommand();
            cmd.Connection = cn;

            cmd.CommandText = @"
                SELECT * 
                FROM Sales.SalesOrderHeader h
                LEFT OUTER JOIN Sales.SalesOrderDetail d 
                    ON h.SalesOrderID = d.SalesOrderID 
                LEFT OUTER JOIN Sales.vSalesPerson p 
                    ON h.SalesPersonID = p.SalesPersonID 
                LEFT OUTER JOIN Sales.vSalesPersonSalesByFiscalYears sfy 
                    ON sfy.SalesPersonID = h.SalesPersonID 
                WHERE h.OrderDate BETWEEN @StartDate AND @EndDate 
                    AND h.SalesPersonID = @SalesPersonID";
            cmd.Parameters.Add("@StartDate", SqlDbType.DateTime);
            cmd.Parameters.Add("@EndDate", SqlDbType.DateTime);
            cmd.Parameters.Add("@SalesPersonID", SqlDbType.Int);

            Console.WriteLine("Running parameterized query {0} times with different parameter values...", QueryIterations);
            long TickCount = Environment.TickCount;
            for (int i = 0; i < QueryIterations; i++)
            {
                cmd.Parameters["@SalesPersonID"].Value = i;
                cmd.Parameters["@StartDate"].Value = DateTime.Parse("2004/01/01", CultureInfo.InvariantCulture);
                cmd.Parameters["@EndDate"].Value = DateTime.Parse("2004/01/10", CultureInfo.InvariantCulture);
                cmd.ExecuteNonQuery();
            }
            TickCount = Environment.TickCount - TickCount;
            Console.WriteLine("\n      Query executions: {0}", QueryIterations);
            Console.WriteLine("  Total execution time: {0} ms", TickCount);
            Console.WriteLine("Average execution time: {0} ms", TickCount / QueryIterations);

            return true;
        }

        static private bool RunQueriesNotParameterized()
        {
            SqlCommand cmd = new SqlCommand();
            cmd.Connection = cn;

            Console.WriteLine("Running non-parameterized query {0} times with different inline literal values...", QueryIterations);
            long TickCount = Environment.TickCount;
            for (int i = 0; i < QueryIterations; i++)
            {
                cmd.CommandText = @"
                    SELECT * 
                    FROM Sales.SalesOrderHeader h
                    LEFT OUTER JOIN Sales.SalesOrderDetail d 
                        ON h.SalesOrderID = d.SalesOrderID 
                    LEFT OUTER JOIN Sales.vSalesPerson p 
                        ON h.SalesPersonID = p.SalesPersonID 
                    LEFT OUTER JOIN Sales.vSalesPersonSalesByFiscalYears sfy 
                        ON sfy.SalesPersonID = h.SalesPersonID 
                    WHERE h.OrderDate BETWEEN '20040101' AND '20040110'  
                        AND h.SalesPersonID = " + i.ToString();
                cmd.ExecuteNonQuery();
            }
            TickCount = Environment.TickCount - TickCount;
            Console.WriteLine("\n      Query executions: {0}", QueryIterations);
            Console.WriteLine("  Total execution time: {0} ms", TickCount);
            Console.WriteLine("Average execution time: {0} ms", TickCount / QueryIterations);

            return true;
        }

        static private void FreeProcCache()
        {
            SqlCommand cmd = new SqlCommand();
            cmd.Connection = cn;
            cmd.CommandText = "DBCC FREEPROCCACHE";
            Console.WriteLine("Clearing procedure cache...");
            cmd.ExecuteNonQuery();
        }

        static private bool ConnectToSql()
        {
            // Connect to the provided server name
            cn.ConnectionString = "App=ParamDemo;Database=AdventureWorks;Persist Security Info=false;";
            if ("" != SqlServer) cn.ConnectionString += "Data Source=" + SqlServer + ";";
            if (""==SqlLogin)
                cn.ConnectionString += "Integrated Security=true;";
            else
                cn.ConnectionString += "Integrated Security=false;User ID=" + SqlLogin 
                    + ";Password=" + SqlPassword;
            Console.WriteLine("Connecting to SQL...");
            try
            {
                cn.Open();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Failed to connect to server '{0}'. Error:\n{1}",
                    SqlServer, ex.Message);
                return false;
            }
            return true;
        }

        static private bool CheckCommandLine(string[] args)
        {
            // Display command line usage 
            if ((args.Length == 0) || (args[0] == "-?") || (args[0] == "/?"))
            {
                DisplayUsage();
                return false;
            }
            // Parse command line arguments
            string ParamType = "";
            string CurrentArg;
            foreach (string arg in args)
            {
                CurrentArg = arg;
                if ((arg.Substring(0,1).Equals("-")) || (arg.Substring(0,1).Equals("-"))) 
                {
                    CurrentArg = arg.Substring(1);
                    if (CurrentArg.Length > 0)
                    {
                        ParamType = CurrentArg.Substring(0, 1);
                        CurrentArg = CurrentArg.Substring(1);
                    }
                    if (0==CurrentArg.Length) continue;
                }
                if (ParamType.Equals("")) goto ParseError;
                switch (ParamType)
                {
                    case "S":
                        SqlServer = CurrentArg;
                        break;
                    case "U":
                        SqlLogin = CurrentArg;
                        break;
                    case "P":
                        SqlPassword = CurrentArg;
                        break;
                    case "I":
                        if (!(UInt32.TryParse (CurrentArg, out QueryIterations))) goto ParseError;
                        break;
                    case "R":
                        if (CurrentArg.Equals("0"))
                            Parameterize = false;
                        else
                            Parameterize = true;
                        break;
                    default:
                        goto ParseError;
                }
                ParamType = "";
            }
            return true;

            ParseError:
                DisplayUsage();
                return false;
        }

        static private void DisplayUsage()
        {
            Console.WriteLine("\nUsage: ParamDemo.exe -S server [-I###] [-R0 | -R1] [-U login -P password]");
            Console.WriteLine("");
            Console.WriteLine("  -S server     SQL Server instance");
            Console.WriteLine("  -U login      SQL Server login (omit for NT authentication)");
            Console.WriteLine("  -P password   SQL Server password (omit for NT authentication)");
            Console.WriteLine("  -I ###        Number of query iterations (default=200)");
            Console.WriteLine("  -R [0|1]      1=Parameterize queries, 0=Do not parameterize queries (default=1)");
            return;
        }
    }
}
