-- 
-- CSS Service Broker Diagnostics and Troubleshooting
--
-- Second Machine file
--

-- The purpose of this script is to showcase the diagnostic 
-- tools available in SQL Server 2005. Be sure to start the 
-- SQL Server Profiler, selecting all of the Broker Event 
-- Category and under Security Audit select Audit Broker 
-- Conversation and Audit Broker Login.

-- Clean up from prior runs & set up for the demo
USE master
GO

DROP DATABASE ssbdiag
GO

drop login [somewhere\machine1$]
go

drop endpoint ep2
go

CREATE DATABASE ssbdiag
GO

-- Now we'll be a bit kinder in this script!
use ssbdiag
GO

CREATE MASTER KEY ENCRYPTION BY PASSWORD = 'we8%rwr$#97we987##%';
CREATE USER sql_ap_user2 WITHOUT LOGIN
CREATE CERTIFICATE sql_ap_certificate2 WITH SUBJECT = 'SQL Applications2', 
	EXPIRY_DATE = '11/11/2011'; 
ALTER AUTHORIZATION ON Certificate::sql_ap_certificate2 TO sql_ap_user2

-- Export the certificate:
BACKUP CERTIFICATE sql_ap_certificate2 TO FILE = 'c:\storedcerts\sql_ap_certificate2';

-- And now back to machine 1...

-- Again we'll set things up properly from the start here:
CREATE CERTIFICATE sql_ap_certificate FROM FILE = 'c:\storedcerts\sql_ap_certificate'
CREATE USER sql_ap_far_user2 WITHOUT LOGIN
ALTER AUTHORIZATION ON Certificate::sql_ap_certificate TO sql_ap_far_user2

-- And the service:
create queue target_queue
create service sql_version on queue target_queue ([DEFAULT])
ALTER AUTHORIZATION ON Service::sql_version TO sql_ap_user2

-- And the endpoint:
create endpoint ep2 state = started as tcp (listener_port = 4022) FOR SERVICE_BROKER ()

-- After a few moments we see the following error in the profiler:
-- Connection handshake failed. The login 'somewhere\machine1$' does not have CONNECT 
-- permission on the endpoint. State 84.
-- 
-- When we created the endpoint, the default is to use windows authentication. We could
-- drop the enpoints and configure them with certificates (like we did with the services)
-- but for now we'll just continue with windows authentication.

-- add in the far machine as a login, and grant connect privilege:
-- Oh, and yes you need to do this in master
use master
create login [somewhere\machine1$] from windows
grant connect on endpoint::ep2 to [somewhere\machine1$]
use ssbdiag

-- now we see in the profiler:
-- An error occurred while receiving data: '10054(An existing connection was forcibly 
-- closed by the remote host.)'.

-- Time to go look at the other machine!

-- Now that we have the permissions granted on the initiating machine we now get the
-- error:
-- This message could not be delivered because the user with ID 6 in database ID 5 does 
-- not have permission to send to the service. Service name: 'sql_version'.

-- Checking this principle:
select * from sys.database_principals where principal_id = 6

-- So we need to allow the far user, access to this service:
GRANT SEND ON SERVICE::sql_version TO sql_ap_far_user2

-- We make a bit more progress, but profiler says:
-- The target service name could not be found. Ensure that the service name is specified 
-- correctly and/or the routing information has been supplied.

-- Note the name in the FileName column: get_sql_version
-- The Broker is trying to respond (acknowledge) back to the initiating service
-- and we have no route to do so...let's add one in:
CREATE ROUTE get_sql_version_route_1
	WITH SERVICE_NAME = N'get_sql_version',
	ADDRESS = 'TCP://machine1.somewhere.com:4022'

-- So profiler is quiet, how about transmission_status?
select * from sys.transmission_queue

-- Hey! How about the service queue?
select * from target_queue

-- Be still my foolish heart!! The message is in the queue!! 
-- OK, so let's reply to the message...
begin tran
declare @message varbinary(4000);
declare @conv uniqueidentifier;
declare @mtid integer;
RECEIVE TOP (1) @conv=conversation_handle, @message=message_body,
	@mtid=message_type_id
	FROM target_queue;
if @conv is NOT NULL
begin
	if @mtid = 2	-- END DIALOG
	begin
		end conversation @conv;
		print 'ending'
	end
	else if @mtid = 1	-- END DIALOG WITH ERROR
	begin
		end conversation @conv;
		print 'ending with error'
	end
	else
	begin
		SEND ON CONVERSATION @conv (@@version);
	end	
end
commit

-- Things look OK in the Profiler and transmission_queue, let's check
-- the other machine...

-- Let's check the queue for the end dialog message:
select * from target_queue

-- And let's run the service code again:
begin tran
declare @message varbinary(4000);
declare @conv uniqueidentifier;
declare @mtid integer;
RECEIVE TOP (1) @conv=conversation_handle, @message=message_body,
	@mtid=message_type_id
	FROM target_queue;
if @conv is NOT NULL
begin
	if @mtid = 2	-- END DIALOG
	begin
		end conversation @conv;
		print 'ending'
	end
	else if @mtid = 1	-- END DIALOG WITH ERROR
	begin
		end conversation @conv;
		print 'ending with error'
	end
	else
	begin
		SEND ON CONVERSATION @conv (@@version);
	end	
end
commit

-- all done!

