-- 
-- CSS Service Broker Diagnostics and Troubleshooting
--
-- First machine file
--

-- The purpose of this script is to showcase the diagnostic 
-- tools available in SQL Server 2005. Be sure to start the 
-- SQL Server Profiler, selecting all of the Broker Event 
-- Category and under Security Audit select Audit Broker 
-- Conversation and Audit Broker Login.


-- Clean up from prior runs & set up for the demo
USE master
GO

DROP DATABASE ssbdiag
go

drop login [somewhere\machine2$]
go

drop endpoint ep1
go

CREATE DATABASE ssbdiag
GO

-- 
-- For the purposes of the demonstration we'll disable the broker
-- in this database. 
alter database ssbdiag set DISABLE_BROKER
go

-- Switch to the new database and begin creating our simple 
-- application
use ssbdiag
go

-- This is the initiating side of our Broker service, so we
-- create a queue for our responses.
create queue initiator_queue

-- We create the service, attach it to the above queue and
-- for this example we'll use the built in "default" contract
-- and message type.
create service get_sql_version on queue initiator_queue ([DEFAULT])

-- Now we'll try to send a message to the "sql_version" service on
-- the other machine.
begin tran
declare @conv uniqueidentifier;
BEGIN DIALOG @conv FROM SERVICE get_sql_version 
	TO SERVICE 'sql_version';
SEND ON CONVERSATION @conv;
commit

-- Now we look at the profiler:
-- Broker:Message Classify - Delayed - The broker is disabled in the sender's database.

-- Let's verify that:
select name, service_broker_guid, is_broker_enabled
	from sys.databases where name = 'ssbdiag'

-- OK, let's enable the Broker:
alter database ssbdiag set ENABLE_BROKER

-- Now we're getting a new event in the profiler:
-- Broker:Message Classify - Delayed - The target service name could not be found. 
-- Ensure that the service name is specified correctly and/or the routing 
-- information has been supplied.

-- OK, let's look at the routes:
select * from sys.routes

-- Yes, we need to put in our route for the service we're targeting:
CREATE ROUTE sql_version_route_1
	WITH SERVICE_NAME = N'sql_version',
	ADDRESS = 'TCP://machine2.somewhere.com:4022'

-- After the above change the Profiler event now reads:
-- Broker:Message Classify - Remote

-- However, things still don't appear to be flowing, so let's look at the transmission
-- queue: 
select * from sys.transmission_queue

-- Specifically the transmission_status column reads:
-- Dialog security is not available for this conversation because there is no 
-- remote service binding for the target service. Create a remote service binding, 
-- or specify ENCRYPTION = OFF in the BEGIN DIALOG statement.

-- Well we want this to be a secure dialog, so we will create a remote service binding.
-- However, we need a user to associate to the target (or far) service:
CREATE USER sql_ap_far_user WITHOUT LOGIN
CREATE REMOTE SERVICE BINDING sql_version_binding_1 
   TO SERVICE 'sql_version' 
   WITH USER = sql_ap_far_user;

-- However, things still don't appear to be flowing, so let's look at the transmission
-- queue: 
select * from sys.transmission_queue

-- Specifically the transmission_status column reads:
-- The session keys for this conversation could not be created or accessed. The database 
-- master key is required for this operation.
-- So let's create the database master key:
CREATE MASTER KEY ENCRYPTION BY PASSWORD = '23987hxJ#KL95234nl0zBe';

-- However, things still don't appear to be flowing, so let's look at the transmission
-- queue: 
select * from sys.transmission_queue

-- Specifically the transmission_status column reads:
-- Dialog security is unavailable for this conversation because there is no security 
-- certificate bound to the database principal (Id: 1). Either create a certificate 
-- for the principal, or specify ENCRYPTION = OFF when beginning the conversation.

-- So who is principal 1?
select * from sys.database_principals where principal_id = 1

-- Well that's "dbo"...why is dbo involved in this conversation? Let's look at the 
-- conversation:
select * from sys.conversation_endpoints
select principal_id, far_principal_id from sys.conversation_endpoints

-- Ahh! This conversation was begun with my (dbo) ID, so I could create a certificate
-- for my ID, but it would be better to create a user specifically for this service:
--
-- DANGER, DANGER WILL ROBINSON!!
-- Do not use CREATE USER FOR CERTIFICATE <-- Only works for assemblies
--
CREATE USER sql_ap_user WITHOUT LOGIN

-- And create the certificate:
CREATE CERTIFICATE sql_ap_certificate WITH SUBJECT = 'SQL Applications', 
	EXPIRY_DATE = '11/11/2011'; 

-- Let's look at that certificate
select * from sys.certificates

-- The pricipal_id is 1 (dbo) and we need to to be associated with our newly created user:
ALTER AUTHORIZATION ON Certificate::sql_ap_certificate TO sql_ap_user

-- Let's look at that certificate again
select * from sys.certificates
select * from sys.database_principals where name = 'sql_ap_user'

-- Yes, it's showing our new principle ID, so now we need to associate this user to the service:
ALTER AUTHORIZATION ON Service::get_sql_version TO sql_ap_user

-- Looking at the service definition now:
select * from sys.services where name = 'get_sql_version'

-- But things still don't appear to be working? 
select * from sys.transmission_queue
-- What about the conversation?
select * from sys.conversation_endpoints
select principal_id, far_principal_id from sys.conversation_endpoints

-- The principal_id is still 1...this conversation is alas doomed, and we need to get 
-- rid of it and start over now that we have a new configuration:
select conversation_handle from sys.conversation_endpoints
end conversation 'C4EFFB73-C06A-DB11-B22B-00065BBD0A16' with cleanup

-- and now let's try this again:
begin tran
declare @conv uniqueidentifier;
BEGIN DIALOG @conv FROM SERVICE get_sql_version 
	TO SERVICE 'sql_version';
SEND ON CONVERSATION @conv;
commit

-- well everthing in the profiler looks OK, what does transmission_status tell us:
select * from sys.transmission_queue

-- Dialog security is unavailable for this conversation because there is no security 
-- certificate bound to the database principal (Id: 5). Either create a certificate 
-- for the principal, or specify ENCRYPTION = OFF when beginning the conversation.

-- And principal 5 is:
select * from sys.database_principals where principal_id = 5

-- Ack! We never set up the certificate for our far end user!
-- Import the cert from the far end...
CREATE CERTIFICATE sql_ap_certificate2 FROM FILE = 'c:\storedcerts\sql_ap_certificate2'
ALTER AUTHORIZATION ON Certificate::sql_ap_certificate2 TO sql_ap_far_user

-- well everthing in the profiler looks OK, what does transmission_status tell us:
select * from sys.transmission_queue

-- transmission_status says:
-- The Service Broker protocol transport is disabled or not configured.

-- Verifying this:
select * from sys.endpoints

-- OK, so let's create an endpoint:
create endpoint ep1 state = started as tcp (listener_port = 4022) FOR SERVICE_BROKER ()

-- Verifying this:
select * from sys.endpoints
select * from sys.service_broker_endpoints

-- Now, I'm going to jump-start this process and send another message so the transport
-- wakes up.
begin tran
declare @conv uniqueidentifier;
BEGIN DIALOG @conv FROM SERVICE get_sql_version 
	TO SERVICE 'sql_version';
SEND ON CONVERSATION @conv;
commit

-- And in the profiler we see a Broker:Connection error:
-- Connection attempt failed with error: '10061(No connection could be made because the 
-- target machine actively refused it.)'.  (Note the transmission_status reflects this too!)

-- And this is true as we've not set up the far end server, but before we do that let's
-- export the certificate we created on this machine:
BACKUP CERTIFICATE sql_ap_certificate TO FILE = 'c:\storedcerts\sql_ap_certificate';

-- Now that we have the far end configured, the profiler here is showing:
-- Connection handshake failed. The login 'somewhere\machine2$' does not have CONNECT 
-- permission on the endpoint. State 84.

-- Yes, same as on the other machine...fixing it:
use master
create login [somewhere\machine2$] from windows
grant connect on endpoint::ep1 to [somewhere\machine2$]
use ssbdiag

-- Things still are not quite right...we see the message: 
-- An error occurred while receiving data: '64(The specified network name is no longer 
-- available.)'.
-- Let's go look at the other machine again

-- If all has gone well we should have our reply to our message in our queue:
select * from initiator_queue

-- And in fact we can see the text of the message_body
select convert(nvarchar(4000),message_body) from initiator_queue

-- So now our client will receive the message and end the dialog...
begin tran
declare @message varbinary(4000);
declare @conv uniqueidentifier;
RECEIVE TOP (1) @conv=conversation_handle, @message=message_body
      FROM initiator_queue;
if @conv is not null
begin
	declare @text nvarchar(4000);
	set @text = @message
	print @text; -- Just for fun...
	end conversation @conv;
end
commit

-- And back to the target machine...


